/*
 * Copyright (c) 2012-2014 Wind River Systems, Inc.
 *
 * SPDX-License-Identifier: Apache-2.0
 */

#include <zephyr/kernel.h>

#include <zephyr/device.h>
#include <zephyr/drivers/gpio.h>
#include <zephyr/drivers/flash.h>
#include <zephyr/drivers/pwm.h>

#include <zephyr/logging/log.h>

#include <string.h>

#define MODULE main
LOG_MODULE_REGISTER(MODULE);

static const struct pwm_dt_spec led_w = PWM_DT_SPEC_GET(DT_NODELABEL(led_w));

static struct k_work_delayable led_work;
static volatile uint8_t breath_cnt = 0;
static void breath_work_fn(struct k_work *w) {
    uint8_t pwm = (breath_cnt > 99) ? (99 - (breath_cnt - 99)) : breath_cnt;
    struct k_work_delayable *dwork;
    dwork = k_work_delayable_from_work(w);
    pwm_set_dt(&led_w, PWM_USEC(100), PWM_USEC(pwm));
    breath_cnt = (breath_cnt >= 198) ? 0 : breath_cnt + 1;
    k_work_reschedule(dwork, K_MSEC(1));
}

static uint8_t wbuf[4096] = {0};
static uint8_t rbuf[4096] = {0};

void main(void) {

    k_sleep(K_MSEC(500));

    LOG_INF("Hello World! %s", CONFIG_BOARD);

    LOG_INF("Flash test");

#if !CONFIG_NORDIC_QSPI_NOR
    const struct device *gpio0 = DEVICE_DT_GET(DT_NODELABEL(gpio0));
    const struct device *gpio1 = DEVICE_DT_GET(DT_NODELABEL(gpio1));
    gpio_pin_configure(gpio0, 22, GPIO_OUTPUT_HIGH);
    gpio_pin_configure(gpio1, 0, GPIO_OUTPUT_HIGH);
#endif

    // initial & start work
    breath_cnt = 0;
    k_work_init_delayable(&led_work, breath_work_fn);

    const struct device *flash = DEVICE_DT_GET(DT_NODELABEL(mx25r64));
    for (size_t i = 0; i < sizeof(wbuf); i++)
        wbuf[i] = i;

    for (uint32_t i = 0;; i++) {
        flash_erase(flash, 0, sizeof(wbuf));
        memset(rbuf, 0, sizeof(rbuf));

        flash_write(flash, 0, wbuf, sizeof(wbuf));
        flash_read(flash, 0, rbuf, sizeof(rbuf));

        if (strncmp(wbuf, rbuf, 4096)) {
            LOG_ERR("flash check error!: %d", i);
        } else {
            if ((i % 10 == 0)) {
                LOG_INF("flash check : %d", i);
            }
        }
    }
}

#include <zephyr/shell/shell.h>
#include <zephyr/shell/shell_uart.h>

static int shell_led_on(const struct shell *shell, size_t argc, char **argv) {
	// Stop work
    k_work_cancel_delayable(&led_work);
    k_work_reschedule(&led_work, K_MSEC(50));
    return 0;
}

static int shell_led_off(const struct shell *shell, size_t argc, char **argv) {
    // Stop work
    k_work_cancel_delayable(&led_work);
    return 0;
}

/* Exported functions --------------------------------------------------------*/
SHELL_STATIC_SUBCMD_SET_CREATE(
    led_cmds,
    SHELL_CMD_ARG(on, NULL, "led on", shell_led_on, 1, 0),
	SHELL_CMD_ARG(off, NULL, "led off", shell_led_off, 1, 0),
    SHELL_SUBCMD_SET_END /* Array terminated. */
);
SHELL_CMD_REGISTER(led, &led_cmds, "Led ON/OFF commands", NULL);
